import requests

BASE_URL = "http://127.0.0.1:5000"

def test_put(session):
    res = requests.put(f"{BASE_URL}/put")
    assert (res.status_code == 200)
    assert res.text=="PUT Request"

def test_put_with_param(session):
    res = requests.put(f"{BASE_URL}/put_with_body", data = {
        "hello": "world"
    })

    assert (res.status_code == 200)
    assert res.text=="hello=world"



from robyn import Robyn, static_file, jsonify, WS
import asyncio
import os
import pathlib
import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

app = Robyn(__file__)
websocket = WS(app, "/web_socket")
i = -1


@websocket.on("message")
async def connect():
    global i
    i += 1
    if i == 0:
        return "Whaaat??"
    elif i == 1:
        return "Whooo??"
    elif i == 2:
        i = -1
        return "*chika* *chika* Slim Shady."


@websocket.on("close")
def close():
    return "GoodBye world, from ws"


@websocket.on("connect")
def message():
    return "Hello world, from ws"


callCount = 0


@app.get("/")
async def hello(request):
    global callCount
    callCount += 1
    _message = "Called " + str(callCount) + " times"
    return jsonify(request)


@app.get("/test/:id")
async def test(request):
    print(request)
    current_file_path = pathlib.Path(__file__).parent.resolve()
    html_file = os.path.join(current_file_path, "index.html")

    return static_file(html_file)


@app.get("/jsonify")
async def json_get():
    return jsonify({"hello": "world"})


@app.get("/query")
async def query_get(request):
    query_data = request["queries"]
    return jsonify(query_data)


@app.post("/jsonify/:id")
async def json(request):
    print(request["params"]["id"])
    return jsonify({"hello": "world"})


@app.post("/post")
async def post():
    return "POST Request"


@app.post("/post_with_body")
async def postreq_with_body(request):
    return bytearray(request["body"]).decode("utf-8")


@app.put("/put")
async def put(request):
    return "PUT Request"


@app.put("/put_with_body")
async def putreq_with_body(request):
    print(request)
    return bytearray(request["body"]).decode("utf-8")


@app.delete("/delete")
async def delete():
    return "DELETE Request"


@app.delete("/delete_with_body")
async def deletereq_with_body(request):
    return bytearray(request["body"]).decode("utf-8")


@app.patch("/patch")
async def patch():
    return "PATCH Request"


@app.patch("/patch_with_body")
async def patchreq_with_body(request):
    return bytearray(request["body"]).decode("utf-8")


@app.get("/sleep")
async def sleeper():
    await asyncio.sleep(5)
    return "sleep function"


@app.get("/blocker")
def blocker():
    import time

    time.sleep(10)
    return "blocker function"


async def startup_handler():
    logger.log(logging.INFO, "Starting up")


@app.shutdown_handler
def shutdown_handler():
    logger.log(logging.INFO, "Shutting down")


if __name__ == "__main__":
    ROBYN_URL = os.getenv("ROBYN_URL", "0.0.0.0")
    app.add_header("server", "robyn")
    current_file_path = pathlib.Path(__file__).parent.resolve()
    os.path.join(current_file_path, "build")
    app.add_directory(
        route="/test_dir",
        directory_path=os.path.join(current_file_path, "build/"),
        index_file="index.html",
    )
    app.startup_handler(startup_handler)
    app.start(port=5000, url=ROBYN_URL)



import requests

BASE_URL = "http://127.0.0.1:5000"


def test_index_request(session):
    res = requests.get(f"{BASE_URL}")
    assert res.status_code == 200


def test_jsonify(session):
    r = requests.get(f"{BASE_URL}/jsonify")
    assert r.json() == {"hello": "world"}
    assert r.status_code == 200


def test_html(session):
    r = requests.get(f"{BASE_URL}/test/123")
    assert "Hello world. How are you?" in r.text


def test_queries(session):
    r = requests.get(f"{BASE_URL}/query?hello=robyn")
    assert r.json() == {"hello": "robyn"}

    r = requests.get(f"{BASE_URL}/query")
    assert r.json() == {}



import requests

BASE_URL = "http://127.0.0.1:5000"

def test_patch(session):
    res = requests.patch(f"{BASE_URL}/patch")
    assert (res.status_code == 200)
    assert res.text=="PATCH Request"

def test_patch_with_param(session):
    res = requests.patch(f"{BASE_URL}/patch_with_body", data = {
        "hello": "world"
    })
    assert (res.status_code == 200)
    assert res.text=="hello=world"



import argparse


class ArgumentParser(argparse.ArgumentParser):
    def __init__(self):
        self.parser = argparse.ArgumentParser(
            description="Robyn, a fast async web framework with a rust runtime."
        )
        self.parser.add_argument(
            "--processes",
            type=int,
            default=1,
            required=False,
            help="Choose the number of processes. [Default: 1]",
        )
        self.parser.add_argument(
            "--workers",
            type=int,
            default=1,
            required=False,
            help="Choose the number of workers. [Default: 1]",
        )
        self.parser.add_argument(
            "--dev",
            dest="dev",
            action="store_true",
            default=False,
            help="Development mode. It restarts the server based on file changes.",
        )

        self.args = self.parser.parse_args()

    def num_processes(self):
        return self.args.processes

    def workers(self):
        return self.args.workers

    def is_dev(self):
        _is_dev = self.args.dev
        if _is_dev and (self.num_processes() != 1 or self.workers() != 1):
            raise Exception("--processes and --workers shouldn't be used with --dev")
        return _is_dev



import asyncio
from inspect import signature


class WS:
    """This is the python wrapper for the web socket that will be used here.
    """
    def __init__(self, robyn_object, endpoint) -> None:
        self.robyn_object = robyn_object
        self.endpoint = endpoint
        self.methods = {}

    def on(self, type):
        def inner(handler):
            if type not in ["connect", "close", "message"]:
                raise Exception(f"Socket method {type} does not exist")
            else:
                self.methods[type] = (handler, self._is_async(handler), self._num_params(handler))
                self.robyn_object.add_web_socket(self.endpoint, self)

        return inner

    def _num_params(self, handler):
        return len(signature(handler).parameters)

    def _is_async(self, handler):
        return asyncio.iscoroutinefunction(handler)




from robyn import Robyn
import os

app = Robyn(__file__)


def test_directory_path():
    assert app.directory_path == os.path.dirname(os.path.abspath(__file__))


def test_file_path():
    assert app.file_path == __file__


